@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
@@ -25,10 +25,31 @@
 #endif
 #endif
 
+#ifdef _MSC_VER
+#include <intrin.h>
+#if (_MSC_VER >= 1600) && !defined(__INTEL_COMPILER)
+#include <immintrin.h>
+#endif
+#endif
+
 static inline int cpuid_ecx(int op)
 {
 #    ifdef _MSC_VER
-#    error "Please write this code and send it to fftw@fftw.org"
+#    ifdef __INTEL_COMPILER
+     int result;
+     _asm {
+	  push rbx
+          mov eax,op
+          cpuid
+          mov result,ecx
+          pop rbx
+     }
+     return result;
+#    else
+     int cpu_info[4];
+     __cpuid(cpu_info,op);
+     return cpu_info[2];
+#    endif
 #    else
      int eax, ecx, edx;
 
@@ -42,7 +63,24 @@ static inline int cpuid_ecx(int op)
 static inline int xgetbv_eax(int op)
 {
 #    ifdef _MSC_VER
-#    error "Please write this code and send it to fftw@fftw.org"
+#    ifdef __INTEL_COMPILER
+     int veax, vedx;
+     _asm {
+          mov ecx,op
+          xgetbv
+          mov veax,eax
+          mov vedx,edx
+     }
+     return veax;
+#    else
+#    if defined(_MSC_VER) && (_MSC_VER >= 1600)
+     unsigned __int64 result;
+     result = _xgetbv(op);
+     return (int)result;
+#    else
+#    error "Need at least Visual Studio 10 SP1 for AVX support"
+#    endif
+#    endif
 #    else
      int eax, edx;
      __asm__ (".byte 0x0f, 0x01, 0xd0" : "=a"(eax), "=d"(edx) : "c" (op));
